/*
* DOM 2.0 Copyright (c) 2000 World Wide Web Consortium,
* (Massachusetts Institute of Technology, Institut National de
* Recherche en Informatique et en Automatique, Keio University). All
* Rights Reserved. This program is distributed under the W3Cs Software
* Intellectual Property License. This program is distributed in the
* hope that it will be useful, but WITHOUT ANY WARRANTY; without even
* the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
* PURPOSE.
* See W3C License http://www.w3.org/Consortium/Legal/ for more details.
*/
// File: http://www.w3.org/TR/2000/REC-DOM-Level-2-Core-20001113/dom.idl


/*
 * C++ Binding - ripotter
 * Copyright 1999-2004 Adobe Systems Incorporated
 * All Rights Reserved.
 *
 * Provided for informational purposes only
*/

// ripotter: May 2002
// Note: This is included in PDDOM for reference only. Original the PDDOM
// classes inherited from these interfaces. However, the resulting
// diamond shape inheritance graphs proved difficult to work with.
// For example: PDDomELement would inherit to versions of the Node
// interface. Even with public virtual inheritance down cast and up cast
// often proved problematic.

#ifndef _H_W_DOM_2_H
#define _H_W_DOM_2_H

//---------------------------------------------------------

typedef PDDomNodeType 	DOMNodeType;

class DocumentType;					// Forward refs
class Document;
class NodeList;
class NamedNodeMap;
class Element;

class PDDOMNode;					// forward ref

//---------------------------------------------------------

typedef enum _EXCEPTIONCODE 
{
    INDEX_SIZE_ERR = 1,
    DOMSTRING_SIZE_ERR,
    HIERARCHY_REQUEST_ERR,
    WRONG_DOCUMENT_ERR,
    INVALID_CHARACTER_ERR,
    NO_DATA_ALLOWED_ERR,
    NO_MODIFICATION_ALLOWED_ERR,
    NOT_FOUND_ERR,
    NOT_SUPPORTED_ERR,
    INUSE_ATTRIBUTE_ERR,
    INVALID_STATE_ERR,
    SYNTAX_ERR,
    INVALID_MODIFICATION_ERR,
    NAMESPACE_ERR,
	INVALID_ACCESS_ERR,       // our (SVG) addition (DOM2.0)
	UNIMPLEMENTED_DOM_ERR,	  // our (SVG) addition
	INVALID_PARAMETER_ERR	  // PDDOM addition
} DOMExceptionCode;

//---------------------------------------------------------

class DOMException 
{
public:
	virtual DOMExceptionCode 	getCode (void) 						= 0;
	virtual void 				setCode (DOMExceptionCode code )	= 0;

	DOMException() {}
	virtual ~DOMException() { }
};

//---------------------------------------------------------

class DOMImplementation 
{
public:
	virtual bool			hasFeature			(const DOMString& feature, const DOMString& version)			= 0;
	virtual DocumentType*	createDocumentType	(const DOMString& qualifiedName, const DOMString& publicId, 
													const DOMString& systemId)									= 0;
	
	virtual Document*		createDocument		(const DOMString& namespaceURI, const DOMString& qualifiedName, 
													const DocumentType& doctype)								= 0;
	
protected:
	DOMImplementation() {}
	virtual ~DOMImplementation() {}
	
private:
	// Functions intentionally private and unimplemented
	DOMImplementation( const DOMImplementation& rhs );				// copy ctor	
	DOMImplementation& operator=( const DOMImplementation& rhs );	// operator =
};

//---------------------------------------------------------

class Node 
{
public:
	virtual ShortDOMString	getNodeName			(void) const							= 0;
	virtual DOMString		getNodeValue		(void) const							= 0;
	virtual void			setNodeValue		(const DOMString&)						= 0;
	virtual DOMNodeType		getNodeType			(void) const							= 0;
	virtual Node*			getParentNode		(void) const							= 0;
	
	virtual NodeList*		getChildNodes		(void)									= 0;
	virtual Node*			getFirstChild		(void) const							= 0;
	virtual Node*			getLastChild		(void) const							= 0;
	virtual Node*			getPreviousSibling	(void) const							= 0;
	virtual Node*			getNextSibling		(void) const							= 0;
	virtual NamedNodeMap*	getAttributes		(void) 									= 0;
	virtual Document*		getOwnerDocument	(void) const							= 0;
	virtual DOMString		getNamespaceURI		(void) const							= 0;
	virtual ShortDOMString	getPrefix			(void) const							= 0;
	virtual void			setPrefix			(const ShortDOMString&)					= 0;
	virtual ShortDOMString	getLocalName		(void) const							= 0;

	virtual Node*			insertBefore		(Node* newChild, Node* refChild)		= 0;
	virtual Node*			replaceChild		(Node* newChild, Node* oldChild)		= 0;
	virtual Node*			removeChild			(Node* oldChild)						= 0;
	virtual Node*			appendChild			(Node* newChild)						= 0;
	virtual bool			hasChildNodes		(void) const							= 0;
	virtual Node*			cloneNode			(bool deep)								= 0;
	virtual void			normalize			(void)									= 0;
	virtual bool			isSupported			\
			(const DOMString& feature, const DOMString& version) const					= 0;
	virtual bool			hasAttributes		(void) const							= 0;

	// PDDomNode extensions needed
	virtual void			addTag				(ASAtom name, void *clientData) const 	= 0;
	virtual void*			getTag				(ASAtom name) const						= 0;
	virtual void*			removeTag			(ASAtom name) const						= 0;

	virtual ~Node() {}

protected:
	Node() {}

private:
	// Functions intentionally private and unimplemented
	Node( const Node& rhs );				// copy ctor		
	Node& operator=( const Node& rhs );		// operator =
};

//---------------------------------------------------------

class NodeList 
{
public:
	virtual Node*			item		(ASUns32 index)	= 0;
	virtual ASUns32			getLength	(void)			= 0;

protected:
	NodeList() {}
	virtual ~NodeList() {}

private:
	// Functions intentionally private and unimplemented
	NodeList( const NodeList& rhs );				// copy ctor		
	NodeList& operator=( const NodeList& rhs );		// operator =
};


//---------------------------------------------------------

class NamedNodeMap 
{
public:
	virtual Node*	getNamedItem		(const ShortDOMString& name) = 0;
	virtual Node*	setNamedItem		(Node* arg) = 0;
	virtual Node*	removeNamedItem		(const ShortDOMString& name) = 0;
	virtual Node*	item				(ASUns32 index) = 0;
	
	virtual ASUns32	getLength			(void) = 0;
	virtual Node*	getNamedItemNS		(const DOMString& namespaceURI, const ShortDOMString& localName) = 0;
	virtual Node*	setNamedItemNS		(Node* arg) = 0;
	virtual Node*	removeNamedItemNS	(const DOMString& namespaceURI, const ShortDOMString& localName) = 0;

	
	virtual ~NamedNodeMap() {}

protected:
	NamedNodeMap() {}

private:
	// Functions intentionally private and unimplemented
	NamedNodeMap( const NamedNodeMap& rhs );				// copy ctor		
	NamedNodeMap& operator=( const NamedNodeMap& rhs );		// operator =
};

//---------------------------------------------------------

class CharacterData : public virtual Node 
{
public:
	virtual DOMString	getData				(void)													= 0;
	virtual void		setData				(const DOMString& newData)								= 0;
	virtual ASUns32		getLength			(void)													= 0;
	virtual DOMString	substringData		(ASUns32 offset, ASUns32 count)							= 0;
	
	virtual void		appendData			(const DOMString& arg)									= 0;
	virtual void		insertData			(ASUns32 offset, const DOMString& arg)					= 0;
	virtual void		deleteData			(ASUns32 offset, ASUns32 count)							= 0;
	virtual void		replaceData			(ASUns32 offset, ASUns32 count, const DOMString& arg)	= 0;
	
protected:
	CharacterData() {}
	virtual ~CharacterData() {}
	
private:
	// Functions intentionally private and unimplemented
	CharacterData( const CharacterData& rhs );				// copy ctor		
	CharacterData& operator=( const CharacterData& rhs );	// operator =
};


//---------------------------------------------------------

class Attr : public virtual Node 
{
public:
	virtual ShortDOMString	getName			(void)	const				= 0;
	virtual bool			getSpecified	(void)	const				= 0;
	virtual DOMString		getValue		(void)	const				= 0;
	virtual void			setValue		(const DOMString& newValue)	= 0;
	virtual Element*		getOwnerElement	(void)						= 0;

protected:
	Attr() {}
	virtual ~Attr() {}
	
private:
	// Functions intentionally private and unimplemented
	Attr( const Attr& rhs );				// copy ctor		
	Attr& operator=( const Attr& rhs );		// operator =
};


//---------------------------------------------------------

class Element : public virtual Node 
{
public:
	virtual ShortDOMString	getTagName				(void) const								= 0;
	virtual DOMString		getAttribute			(const ShortDOMString& name)				= 0;
	virtual void			setAttribute			(const ShortDOMString& name, const DOMString& value) = 0;
	virtual void			removeAttribute			(const ShortDOMString& name)				= 0;
	virtual Attr*			getAttributeNode		(const ShortDOMString& name)				= 0;
	virtual Attr*			setAttributeNode		(Attr& newAttr)								= 0;
	virtual Attr*			removeAttributeNode		(Attr& oldAttr)								= 0;
	virtual NodeList*		getElementsByTagName	(const ShortDOMString& name)				= 0;
	
	// Introduced in DOM Level 2:
	virtual DOMString		getAttributeNS			(const DOMString& namespaceURI, 
															const ShortDOMString& localName)	= 0;
	virtual void			setAttributeNS			(const DOMString& namespaceURI,
															const DOMString& qualifiedName,
															const DOMString& value)				= 0;
	
	virtual void			removeAttributeNS		(const DOMString& namespaceURI, 
															const ShortDOMString& localName)			= 0;

	virtual Attr*			getAttributeNodeNS		(const DOMString& namespaceURI, 
															const ShortDOMString& localName)			= 0;
	virtual Attr*			setAttributeNodeNS		(Attr& newAttr)								= 0;
	virtual NodeList*		getElementsByTagNameNS	(const DOMString& namespaceURI,
															const ShortDOMString& localName)	= 0;
	virtual bool			hasAttribute			(const ShortDOMString& name)				= 0;
	virtual bool			hasAttributeNS			(const DOMString& namespaceURI, 
															const ShortDOMString& localName)	= 0;

protected:
	Element() {}
	virtual ~Element() {}
	
private:
	// Functions intentionally private and unimplemented
	Element( const Element& rhs );				// copy ctor		
	Element& operator=( const Element& rhs );	// operator =
};

//----------------------------------------------------------------------------

class Text : public CharacterData
{
public:
	virtual Text*		splitText	(ASUns32 offset) = 0;

protected:
	Text() {}
	virtual ~Text() {}

private:
	// Functions intentionally private and unimplemented
	Text( const Text&  );					// copy ctor		
	Text& operator=( const Text&  );		// operator =
};

//----------------------------------------------------------------------------

class Comment : public CharacterData
{
protected:
	Comment() {}
	virtual ~Comment() {}

private:
	// Functions intentionally private and unimplemented
	Comment( const Comment&  );					// copy ctor		
	Comment& operator=( const Comment&  );		// operator =
};

//----------------------------------------------------------------------------

class CDATASection : public Text
{
protected:
	CDATASection() {}
	virtual ~CDATASection() {}

private:
	// Functions intentionally private and unimplemented
	CDATASection( const CDATASection&  );					// copy ctor		
	CDATASection& operator=( const CDATASection&  );		// operator =
};

//----------------------------------------------------------------------------

class DocumentType : public virtual Node 
{
public:
	virtual DOMString		getName				(void) = 0;
	virtual NamedNodeMap*	getEntities			(void) = 0;
	virtual NamedNodeMap*	getNotations		(void) = 0;
	virtual DOMString		getPublicId			(void) = 0;
	virtual DOMString		getSystemId			(void) = 0;
	virtual DOMString		getInternalSubset 	(void) = 0;

protected:
	DocumentType() {}
	virtual ~DocumentType() {}

private:
	// Functions intentionally private and unimplemented
	DocumentType( const DocumentType&  );					// copy ctor		
	DocumentType& operator=( const DocumentType&  );		// operator =
};

//----------------------------------------------------------------------------

class Notation : public virtual Node 
{
public:
	virtual DOMString getPublicId	(void)	= 0;
	virtual DOMString getSystemId	(void)	= 0;

protected:
	Notation() {}
	virtual ~Notation() {}

private:
	// Functions intentionally private and unimplemented
	Notation( const Notation&  );					// copy ctor		
	Notation& operator=( const Notation&  );		// operator =
};

//----------------------------------------------------------------------------

class Entity : public virtual Node 
{
public:
	virtual DOMString getPublicId			(void)	= 0;
	virtual DOMString getSystemId			(void)	= 0;
	virtual DOMString getNotationName		(void)	= 0;

protected:
	Entity() {}
	virtual ~Entity() {}

private:
	// Functions intentionally private and unimplemented
	Entity( const Entity&  );					// copy ctor		
	Entity& operator=( const Entity&  );		// operator =
};

//----------------------------------------------------------------------------

class EntityReference : public virtual Node 
{
protected:
	EntityReference() {}
	virtual ~EntityReference() {}

private:
	// Functions intentionally private and unimplemented
	EntityReference( const EntityReference&  );				// copy ctor		
	EntityReference& operator=( const EntityReference&  );		// operator =
};

//----------------------------------------------------------------------------

class ProcessingInstruction : public virtual Node 
{
public:
	virtual DOMString	getTarget		(void)				= 0;
	virtual DOMString	getData			(void)				= 0;
	virtual void		setData			(const DOMString&)	= 0;

protected:
	ProcessingInstruction() {}
	virtual ~ProcessingInstruction() {}

private:
	// Functions intentionally private and unimplemented
	ProcessingInstruction( const ProcessingInstruction& rhs );					// copy ctor		
	ProcessingInstruction& operator=( const ProcessingInstruction& rhs );		// operator =
};


//----------------------------------------------------------------------------

class DocumentFragment : public virtual Node
{
protected:
	DocumentFragment() {}
	virtual ~DocumentFragment() {}

private:
	// Functions intentionally private and unimplemented
	DocumentFragment( const DocumentFragment& rhs );				// copy ctor		
	DocumentFragment& operator=( const DocumentFragment& rhs );		// operator =
};

//----------------------------------------------------------------------------

class Document : public virtual Node 
{
	virtual DocumentType*		getDoctype					(void) const							= 0;
	virtual DOMImplementation*	getImplementation			(void)									= 0;
	virtual Element*			getDocumentElement			(void)									= 0;
	virtual Element*			createElement				(const ShortDOMString& tagName)			= 0;
	virtual DocumentFragment*	createDocumentFragment		(void)									= 0;
	virtual Text*				createTextNode				(const DOMString& data)					= 0;
	virtual Comment*			createComment				(const DOMString& data)					= 0;
	virtual CDATASection*		createCDATASection			(const DOMString& data)					= 0;
	virtual ProcessingInstruction*
								createProcessingInstruction	(const DOMString& target, const DOMString& data) = 0;
	virtual Attr*				createAttribute				(const ShortDOMString& name)			= 0;
	virtual EntityReference*	createEntityReference		(const DOMString& name)					= 0;
	virtual NodeList*			getElementsByTagName		(const ShortDOMString& tagname)			= 0;

	// Introduced in DOM Level 2:
	virtual Node*				importNode					(Node& importedNode, bool deep)			= 0;
	virtual Element*			createElementNS				(const DOMString& namespaceURI, 
																	const DOMString& qualifiedName) = 0;
	virtual Attr*				createAttributeNS			(const DOMString& namespaceURI, 
																	const DOMString& qualifiedName)	= 0;
	virtual NodeList*			getElementsByTagNameNS		(const DOMString& namespaceURI, 
																	const ShortDOMString& localName)= 0;
	virtual Element*			getElementById				(const DOMString& elementId)			= 0;

protected:
	Document() {}
	virtual ~Document() {}

private:
	// Functions intentionally private and unimplemented
	Document( const Document& rhs );				// copy ctor		
	Document& operator=( const Document& rhs );		// operator =
};


// gfshgfjsfds
#endif // _H_W_DOM_2_H
