/*
 * Copyright (c) 2000 World Wide Web Consortium,
 * (Massachusetts Institute of Technology, Institut National de
 * Recherche en Informatique et en Automatique, Keio University). All
 * Rights Reserved. This program is distributed under the W3C's Software
 * Intellectual Property License. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.
 * See W3C License http://www.w3.org/Consortium/Legal/ for more details.
 */

/*
 * C++ Binding - ripotter
 * Copyright 1999-2004 Adobe Systems Incorporated
 * All Rights Reserved.
 *
 * Provided for informational purposes only
*/

#ifndef _H_DOM2_EVENTS_H
#define _H_DOM2_EVENTS_H

// In DOM this is of type DOMString
// We have a much more limited event type.
typedef PDDomEventType	EventType;

class EventListener;
class Event;
class AbstractView;			// Dependency on DOM 2.0 Views

typedef PDDomEventPhase	DomEventPhase;
typedef PDDomEventType	DomEventType;

typedef enum _EVENT_EXCEPTIONCODE 
{
    UNSPECIFIED_EVENT_TYPE_ERR = 0
} 
EventExceptionCode;

//-------------------------------------------------------------------

class EventException 
{
public:
	virtual EventException 	getCode	(void) 						= 0;
	virtual void 			setCode	(EventExceptionCode code)	= 0;

	EventException() {}
	virtual ~EventException() { }  
};
  
//-------------------------------------------------------------------

class EventTarget 
{
public:
    virtual void	addEventListener	
    			(DomEventType type, EventListener* listener, bool useCapture) = 0;
    			
    virtual void	removeEventListener
    			(DomEventType type, EventListener* listener, bool useCapture) = 0;
    			
    virtual bool	dispatchEvent	(Event* evt) = 0;
 
protected:
	EventTarget() {}
	virtual ~EventTarget() {}
	
private:
	// Functions intentionally private and unimplemented
	EventTarget( const EventTarget& rhs );					// copy ctor	
	EventTarget& operator=( const EventTarget& rhs );		// operator =
};

//-------------------------------------------------------------------

class EventListener 
{
public:
	virtual void	handleEvent		(Event* evt) = 0;
	
protected:
	EventListener() {}
	virtual ~EventListener() {}
	
private:
	// Functions intentionally private and unimplemented
	EventListener( const EventListener& rhs );					// copy ctor	
	EventListener& operator=( const EventListener& rhs );		// operator =
};

//-------------------------------------------------------------------

class Event 
{
public:
    virtual DomEventType	getType				(void)	= 0;
    virtual EventTarget*    getTarget			(void)	= 0;
    virtual EventTarget*	getCurrentTarget	(void) 	= 0;
   	virtual DomEventPhase	getEventPhase		(void) 	= 0;
    virtual bool    	     getBubbles			(void)	= 0;
    virtual bool  	      	getCancelable		(void)	= 0;
    virtual DOMTimeStamp	getTimeStamp		(void)	= 0;
    
    virtual	void			stopPropagation		(void)	= 0;
    virtual void			preventDefault		(void)	= 0;
    
    virtual void			initEvent			(DomEventType eventTypeArg,  bool canBubbleArg, bool cancelableArg) = 0;

protected:
	Event() {}
	virtual ~Event() {}
	
private:
	// Functions intentionally private and unimplemented
	Event(const Event& rhs);					// copy ctor	
	Event& operator=( const Event& rhs );		// operator =
};



//-------------------------------------------------------------------

class DocumentEvent 
{
public:
    Event*	createEvent	(DomEventType eventType);
    
protected:
	DocumentEvent() {}
	virtual ~DocumentEvent() {}
	
private:
	// Functions intentionally private and unimplemented
	DocumentEvent(const DocumentEvent& rhs);					// copy ctor	
	DocumentEvent& operator=( const DocumentEvent& rhs );		// operator =
};



//-------------------------------------------------------------------

class UIEvent : public Event 
{
public:
	// Dependency on views
	virtual AbstractView*	getView		(void) 		= 0;
	virtual ASFixed			getDetail	(void)		= 0;
	virtual void			initUIEvent	(DomEventType typeArg, 
                              	 			bool canBubbleArg, bool cancelableArg, 
                              				AbstractView* viewArg, ASFixed detailArg) = 0;

protected:
	UIEvent() {}
	virtual ~UIEvent() {}
	
private:
	// Functions intentionally private and unimplemented
	UIEvent(const UIEvent& rhs);					// copy ctor	
	UIEvent& operator=( const UIEvent& rhs );		// operator =
};



//-------------------------------------------------------------------

class MouseEvent : UIEvent 
{
public:
	virtual ASFixed             getScreenX		(void)	= 0;
	virtual ASFixed             getScreenY		(void)	= 0;
	virtual ASFixed             getClientX		(void)	= 0;
	virtual ASFixed             getClientY		(void)	= 0;
	virtual bool				getCtrlKey		(void)	= 0;
	virtual bool				getShiftKey		(void)	= 0;
	virtual bool				getAltKey		(void)	= 0;
	virtual bool				getMetaKey		(void)	= 0;
	virtual ASUns32				getButton		(void)	= 0;
	virtual EventTarget*		getRelatedTarget(void)	= 0;
	
	virtual void				initMouseEvent	(DomEventType		typeArg, 
				                                  bool 				canBubbleArg, 
				                                  bool 				cancelableArg, 
				                                  AbstractView*		viewArg, 
				                                  const ASFixed*	detailArg, 
				                                  const ASFixed*	screenXArg, 
				                                  const ASFixed*	screenYArg, 
				                                  const ASFixed*	clientXArg, 
				                                  const ASFixed*	clientYArg, 
				                                  bool 				ctrlKeyArg, 
				                                  bool 				altKeyArg, 
				                                  bool 				shiftKeyArg, 
				                                  bool 				metaKeyArg, 
				                                  ASUns32			buttonArg, 
				                                  EventTarget*		relatedTargetArg) = 0;
protected:
	MouseEvent() {}
	virtual ~MouseEvent() {}
	
private:
	// Functions intentionally private and unimplemented
	MouseEvent(const MouseEvent& rhs);					// copy ctor	
	MouseEvent& operator=( const MouseEvent& rhs );		// operator =				                                  
};



//-------------------------------------------------------------------

class MutationEvent : Event 
{
public:
	virtual Node*            	getRelatedNode		(void)	= 0;
	virtual DOMString        	getPrevValue		(void)	= 0;
	virtual DOMString        	getNewValue			(void)	= 0;
	virtual ShortDOMString      getAttrName			(void)	= 0;
	virtual DomAttrChangeType	getAttrChange		(void)	= 0;
	virtual void				initMutationEvent	(DomEventType			typeArg, 
				                                     bool 					canBubbleArg, 
				                                     bool	 				cancelableArg, 
				                                     Node* 					relatedNodeArg, 
				                                     const DOMString&		prevValueArg, 
				                                     const DOMString&		newValueArg, 
				                                     const ShortDOMString&	attrNameArg, 
				                                     DomAttrChangeType		attrChangeArg);
protected:
	MutationEvent() {}
	virtual ~MutationEvent() {}
	
private:
	// Functions intentionally private and unimplemented
	MutationEvent(const MutationEvent& rhs);					// copy ctor	
	MutationEvent& operator=( const MutationEvent& rhs );		// operator =	
};

#endif // _EVENTS_IDL_

