/*
 * Taken from IDL Binding
 * Copyright (c) 2000 World Wide Web Consortium,
 * (Massachusetts Institute of Technology, Institut National de
 * Recherche en Informatique et en Automatique, Keio University). All
 * Rights Reserved. This program is distributed under the W3C's Software
 * Intellectual Property License. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.
 * See W3C License http://www.w3.org/Consortium/Legal/ for more details.
 * File: http://www.w3.org/TR/2000/REC-DOM-Level-2-Traversal-Range-20001113/traversal.idl
 */
 
/*
 * C++ Binding - ripotter
 * Copyright 1999-2004 Adobe Systems Incorporated
 * All Rights Reserved.
 *
 * Provided for informational purposes only
*/

#ifndef _H_DOM_TRAVERSAL_2_H
#define _H_DOM_TRAVERSAL_2_H

class NodeFilter; // Forward Ref
typedef PDDomNodeFilterResponse	DomNodeFilterResponse;

//-------------------------------------------------------------------

class NodeIterator 
{
public:
	virtual Node*		getRoot						(void)		= 0;
	virtual ASUns32		getWhatToShow				(void)		= 0;
	virtual NodeFilter*	getFilter					(void)		= 0;
	virtual bool		getExpandEntityReferences	(void)		= 0;
	
	virtual Node*		nextNode					(void)		= 0;
	virtual Node*		previousNode				(void)		= 0;
	virtual void		detach						(void)		= 0;

protected:
	NodeIterator() {}
	virtual ~NodeIterator() {}
	
private:
	// Functions intentionally private and unimplemented
	NodeIterator( const NodeIterator& rhs );				// copy ctor	
	NodeIterator& operator=( const NodeIterator& rhs );		// operator =
};



//-------------------------------------------------------------------

class NodeFilter 
{
public:
	virtual DomNodeFilterResponse 	acceptNode		(Node* n) = 0;
	
protected:
	NodeFilter() {}
	virtual ~NodeFilter() {}
	
private:
	// Functions intentionally private and unimplemented
	NodeFilter( const NodeFilter& rhs );				// copy ctor	
	NodeFilter& operator=( const NodeFilter& rhs );		// operator =
};



//-------------------------------------------------------------------

class TreeWalker 
{
public:
	virtual Node*		getRoot						(void)		= 0;
	virtual ASUns32		getWhatToShow				(void)		= 0;
	virtual NodeFilter*	getFilter					(void)		= 0;
	virtual bool		getExpandEntityReferences	(void)		= 0;
	
    virtual Node*		getCurrentNode				(void)		= 0;
    virtual Node*		setCurrentNode				(Node* n)	= 0;

	virtual Node*		parentNode					(void)		= 0;
	virtual Node*		firstChild					(void)		= 0;
	virtual Node*		lastChild					(void)		= 0;
	virtual Node*		previousSibling				(void)		= 0;
	virtual Node*		nextSibling					(void)		= 0;
	virtual Node*		previousNode				(void)		= 0;
	virtual Node*		nextNode					(void)		= 0;

protected:
	TreeWalker() {}
	virtual ~TreeWalker() {}
	
private:
	// Functions intentionally private and unimplemented
	TreeWalker( const TreeWalker& rhs );				// copy ctor	
	TreeWalker& operator=( const TreeWalker& rhs );		// operator =
};



//-------------------------------------------------------------------

class DocumentTraversal 
{
public:
	virtual NodeIterator*	createNodeIterator
			(Node* root, ASUns32 whatToShow, NodeFilter* filter, 
                                          bool entityReferenceExpansion)	= 0;
                                          
    virtual TreeWalker*		createTreeWalker
    		(Node* root, ASUns32 whatToShow, NodeFilter* filter, 
                                        bool entityReferenceExpansion) = 0;
                   
protected:
	DocumentTraversal() {}
	virtual ~DocumentTraversal() {}
	
private:
	// Functions intentionally private and unimplemented
	DocumentTraversal( const DocumentTraversal& rhs );				// copy ctor	
	DocumentTraversal& operator=( const DocumentTraversal& rhs );	// operator =
};

#endif // _H_DOM_TRAVERSAL_2_H





